@@ -1,71 +1,50 @@
 
 #include "searchengine.h"
 
-#include "stdlib.h"
-
-#include <QTextDocument>
-#include <QLocale>
-#include <KConfig>
-#include <KProcess>
-#include <KShell>
 #include <QMessageBox>
 
 #include "docmetainfo.h"
-#include "formatter.h"
 #include "view.h"
 #include "searchhandler.h"
-#include "prefs.h"
+#include "khc_debug.h"
+#include "grantleeformatter.h"
+
+#include <prefs.h>
 
 namespace KHC
 {
 
 SearchTraverser::SearchTraverser( SearchEngine *engine, int level ) :
-  mMaxLevel( 999 ), mEngine( engine), mLevel( level )
+  mMaxLevel( 999 ), mEngine( engine), mLevel( level ), mResultsPtr( &mResults )
 {
 #if 0
-  kDebug() << "SearchTraverser(): " << mLevel
-    << "  0x" << QString::number( int( this ), 16 ) << endl;
+  khcDebug() << "SearchTraverser():" << mLevel << this;
 #endif
 }
 
 SearchTraverser::~SearchTraverser()
 {
 #if 0
-    kDebug() << "~SearchTraverser(): " << mLevel
-      << "  0x" << QString::number( int( this ), 16 ) << endl;
+    khcDebug() << "~SearchTraverser():" << mLevel << this;
 #endif
-
-  QString section;
-  if ( parentEntry() ) {
-    section = parentEntry()->name();
-  } else {
-    section = ("Unknown Section");
-  }
-
-  if ( !mResult.isEmpty() ) {
-    mEngine->view()->writeSearchResult(
-      mEngine->formatter()->sectionHeader( section ) );
-    mEngine->view()->writeSearchResult( mResult );
-  }
 }
 
 void SearchTraverser::process( DocEntry * )
 {
-  qDebug() << "SearchTraverser::process()";
+  khcDebug() << "SearchTraverser::process()";
 }
 
 void SearchTraverser::startProcess( DocEntry *entry )
 {
-//  kDebug() << "SearchTraverser::startProcess(): " << entry->name() << "  "
-//    << "SEARCH: '" << entry->search() << "'" << endl;
+//  khcDebug() << "SearchTraverser::startProcess():" << entry->name()
+//    << "SEARCH:" << entry->search();
 
   if ( !mEngine->canSearch( entry ) || !entry->searchEnabled() ) {
     mNotifyee->endProcess( entry, this );
     return;
   }
 
-//  kDebug() << "SearchTraverser::startProcess(): " << entry->identifier()
-//    << endl;
+//  khcDebug() << "SearchTraverser::startProcess():" << entry->identifier();
 
   SearchHandler *handler = mEngine->handler( entry->documentType() );
 
@@ -86,7 +65,7 @@ void SearchTraverser::startProcess( DocEntry *entry )
   handler->search( entry, mEngine->words(), mEngine->maxResults(),
     mEngine->operation() );
 
-//  kDebug() << "SearchTraverser::startProcess() done: " << entry->name();
+//  khcDebug() << "SearchTraverser::startProcess() done: " << entry->name();
 }
 
 void SearchTraverser::connectHandler( SearchHandler *handler )
@@ -109,8 +88,7 @@ void SearchTraverser::disconnectHandler( SearchHandler *handler )
   QMap<SearchHandler *,int>::Iterator it;
   it = mConnectCount.find( handler );
   if ( it == mConnectCount.end() ) {
-    qWarning() << "SearchTraverser::disconnectHandler() handler not connected."
-      << endl;
+    khcWarning() << "SearchTraverser::disconnectHandler() handler not connected.";
   } else {
     int count = *it;
     --count;
@@ -126,13 +104,14 @@ void SearchTraverser::disconnectHandler( SearchHandler *handler )
 
 DocEntryTraverser *SearchTraverser::createChild( DocEntry *parentEntry )
 {
-//  kDebug() << "SearchTraverser::createChild() level " << mLevel;
+//  khcDebug() << "SearchTraverser::createChild() level " << mLevel;
 
   if ( mLevel >= mMaxLevel ) {
     ++mLevel;
     return this;
   } else {
-    DocEntryTraverser *t = new SearchTraverser( mEngine, mLevel + 1 );
+    SearchTraverser *t = new SearchTraverser( mEngine, mLevel + 1 );
+    t->mResultsPtr = mResultsPtr;
     t->setParentEntry( parentEntry );
     return t;
   }
@@ -140,7 +119,7 @@ DocEntryTraverser *SearchTraverser::createChild( DocEntry *parentEntry )
 
 DocEntryTraverser *SearchTraverser::parentTraverser()
 {
-//  kDebug() << "SearchTraverser::parentTraverser(): level: " << mLevel;
+//  khcDebug() << "SearchTraverser::parentTraverser(): level: " << mLevel;
 
   if ( mLevel > mMaxLevel ) {
     return this;
@@ -151,7 +130,7 @@ DocEntryTraverser *SearchTraverser::parentTraverser()
 
 void SearchTraverser::deleteTraverser()
 {
-//  kDebug() << "SearchTraverser::deleteTraverser()";
+//  khcDebug() << "SearchTraverser::deleteTraverser()";
 
   if ( mLevel > mMaxLevel ) {
     --mLevel;
@@ -162,11 +141,9 @@ void SearchTraverser::deleteTraverser()
 
 void SearchTraverser::showSearchError( SearchHandler *handler, DocEntry *entry, const QString &error )
 {
-//  kDebug() << "SearchTraverser::showSearchError(): " << entry->name()
-//    << endl;
+//  khcDebug() << "SearchTraverser::showSearchError():" << entry->name();
 
-  mResult += mEngine->formatter()->docTitle( entry->name() );
-  mResult += mEngine->formatter()->paragraph( error );
+  mResultsPtr->append( qMakePair( entry, error ) );
 
   mEngine->logError( entry, error );
 
@@ -177,11 +154,9 @@ void SearchTraverser::showSearchError( SearchHandler *handler, DocEntry *entry,
 
 void SearchTraverser::showSearchResult( SearchHandler *handler, DocEntry *entry, const QString &result )
 {
-//  kDebug() << "SearchTraverser::showSearchResult(): " << entry->name()
-//    << endl;
+//  khcDebug() << "SearchTraverser::showSearchResult():" << entry->name();
 
-  mResult += mEngine->formatter()->docTitle( entry->name() );
-  mResult += mEngine->formatter()->processResult( result );
+  mResultsPtr->append( qMakePair( entry, result ) );
 
   disconnectHandler( handler );
 
@@ -190,21 +165,26 @@ void SearchTraverser::showSearchResult( SearchHandler *handler, DocEntry *entry,
 
 void SearchTraverser::finishTraversal()
 {
-//  kDebug() << "SearchTraverser::finishTraversal()";
+//  khcDebug() << "SearchTraverser::finishTraversal()";
 
-  mEngine->view()->writeSearchResult( mEngine->formatter()->footer() );
+  mEngine->view()->beginSearchResult();
+  mEngine->view()->writeSearchResult( mEngine->grantleeFormatter()->formatSearchResults( mWords, mResults ) );
   mEngine->view()->endSearchResult();
 
   mEngine->finishSearch();
 }
 
+void SearchTraverser::setWords( const QString &words )
+{
+  mWords = words;
+}
+
 
 SearchEngine::SearchEngine( View *destination )
   : QObject(),
-    mProc( 0 ), mSearchRunning( false ), mView( destination ),
+    mSearchRunning( false ), mView( destination ),
     mRootTraverser( 0 )
 {
-  mLang = QLocale().bcp47Name().left( 2 );
 }
 
 SearchEngine::~SearchEngine()
@@ -227,7 +207,7 @@ bool SearchEngine::initSearchHandlers()
   QStringList::ConstIterator it;
   for( it = resources.constBegin(); it != resources.constEnd(); ++it ) {
     QString filename = *it;
-    qDebug() << "SearchEngine::initSearchHandlers(): " << filename;
+    khcDebug() << "SearchEngine::initSearchHandlers(): " << filename;
     SearchHandler *handler = SearchHandler::initFromFile( filename );
     if ( !handler ) {
      QMessageBox::warning( mView->widget(), i18n("Handler"), i18n("Unable to initialize SearchHandler from file '%1'.", filename ) );
@@ -235,14 +215,23 @@ bool SearchEngine::initSearchHandlers()
       QStringList documentTypes = handler->documentTypes();
       QStringList::ConstIterator it;
       for( it = documentTypes.constBegin(); it != documentTypes.constEnd(); ++it ) {
-        mHandlers.insert( *it, handler );
+        // only register "handler" for this content type if there is no
+        // handler set already; since we read more specific directories
+        // first, then those will override whatever is in more global
+        // ones
+        if ( !mHandlers.contains( *it ) ) {
+          mHandlers.insert( *it, handler );
+        } else {
+          // not keeping "handler", delete it
+          delete handler;
+        }
       }
     }
   }
 
   if ( mHandlers.isEmpty() ) {
     QString txt = i18n("No valid search handler found.");
-    qWarning() << txt;
+    khcWarning() << txt;
 //    KMessageBox::sorry( mView->widget(), txt );
     return false;
   }
@@ -250,16 +239,8 @@ bool SearchEngine::initSearchHandlers()
   return true;
 }
 
-void SearchEngine::searchExited(int exitCode, QProcess::ExitStatus exitStatus)
-{
-  Q_UNUSED(exitCode);
-  Q_UNUSED(exitStatus);
-  qDebug() << "Search terminated";
-  mSearchRunning = false;
-}
-
 bool SearchEngine::search( const QString & words, const QString & method, int matches,
-                           const QString & scope )
+                           const QStringList & scope )
 {
   if ( mSearchRunning ) return false;
 
@@ -275,111 +256,25 @@ bool SearchEngine::search( const QString & words, const QString & method, int ma
   if ( method == "or" ) mOperation = Or;
   else mOperation = And;
 
-  KConfigGroup cfg(KSharedConfig::openConfig(), "Search");
-  QString commonSearchProgram = cfg.readPathEntry( "CommonProgram", QString() );
-  bool useCommon = cfg.readEntry( "UseCommonProgram", false);
-
-  if ( commonSearchProgram.isEmpty() || !useCommon ) {
-    if ( !mView ) {
-      return false;
-    }
-
-    QString txt = i18n("Search Results for '%1':", words.toHtmlEscaped() );
-
-    mStderr = "<b>" + txt + "</b>\n";
-
-    mView->beginSearchResult();
-    mView->writeSearchResult( formatter()->header( i18n("Search Results") ) );
-    mView->writeSearchResult( formatter()->title( txt ) );
-
-    if ( mRootTraverser ) {
-      qDebug() << "SearchEngine::search(): mRootTraverser not null.";
-      return false;
-    }
-    mRootTraverser = new SearchTraverser( this, 0 );
-    DocMetaInfo::self()->startTraverseEntries( mRootTraverser );
-
-    return true;
-  } else {
-    QString lang = QLocale().bcp47Name().left( 2 );
-
-    if ( lang.toLower() == "c" || lang.toLower() == "posix" )
-	  lang = "en";
-
-    // if the string contains '&' replace with a '+' and set search method to and
-    if (mWords.indexOf("&") != -1) {
-      mWords.replace('&', ' ');
-      mMethod = "and";
-    }
-
-    // replace whitespace with a '+'
-    mWords = mWords.trimmed();
-    mWords = mWords.simplified();
-    mWords.replace(QRegExp("\\s"), "+");
-
-    commonSearchProgram = substituteSearchQuery( commonSearchProgram );
-
-    qDebug() << "Common Search: " << commonSearchProgram;
-
-    mProc = new KProcess();
-    *mProc << KShell::splitArgs(commonSearchProgram);
-
-    connect( mProc, SIGNAL( finished(int, QProcess::ExitStatus) ),
-             this, SLOT( searchExited(int, QProcess::ExitStatus) ) );
-
-    mSearchRunning = true;
-    mSearchResult.clear();
-    mStderr = QStringLiteral("<b>") + commonSearchProgram + QStringLiteral("</b>\n\n");
-
-    mProc->start();
-    if (!mProc->waitForStarted()) {
-      qWarning() << "could not start search program '" << commonSearchProgram << "'";
-      delete mProc;
-      return false;
-    }
-
-    while (mSearchRunning && mProc->state() == QProcess::Running)
-      qApp->processEvents();
-
-    // no need to use signals/slots
-    mStderr += mProc->readAllStandardError();
-    mSearchResult += mProc->readAllStandardOutput();
-
-    if ( mProc->exitStatus() == KProcess::CrashExit || mProc->exitCode() != 0 ) {
-      qWarning() << "Unable to run search program '" << commonSearchProgram << "'" << endl;
-      delete mProc;
-
-      return false;
-    }
-
-    delete mProc;
-
-    // modify the search result
-    mSearchResult = mSearchResult.replace("http://localhost/", "file:/");
-    mSearchResult = mSearchResult.mid( mSearchResult.indexOf( '<' ) );
-
-    mView->beginSearchResult();
-    mView->writeSearchResult( mSearchResult );
-    mView->endSearchResult();
-
-    emit searchFinished();
+  if ( !mView ) {
+    return false;
   }
 
+  QString txt = i18n("Search Results for '%1':", words.toHtmlEscaped() );
+
+  mStderr = "<b>" + txt + "</b>\n";
+
+  if ( mRootTraverser ) {
+    khcDebug() << "SearchEngine::search(): mRootTraverser not null.";
+    return false;
+  }
+  mRootTraverser = new SearchTraverser( this, 0 );
+  mRootTraverser->setWords( words );
+  DocMetaInfo::self()->startTraverseEntries( mRootTraverser );
+
   return true;
 }
 
-QString SearchEngine::substituteSearchQuery( const QString &query )
-{
-  QString result = query;
-  result.replace( QLatin1String("%k"), mWords );
-  result.replace( QLatin1String("%n"), QString::number( mMatches ) );
-  result.replace( QLatin1String("%m"), mMethod );
-  result.replace( QLatin1String("%l"), mLang );
-  result.replace( QLatin1String("%s"), mScope );
-
-  return result;
-}
-
 QString SearchEngine::substituteSearchQuery( const QString &query,
   const QString &identifier, const QStringList &words, int maxResults,
   Operation operation, const QString &lang, const QString& binary )
@@ -397,9 +292,9 @@ QString SearchEngine::substituteSearchQuery( const QString &query,
   return result;
 }
 
-Formatter *SearchEngine::formatter() const
+GrantleeFormatter *SearchEngine::grantleeFormatter() const
 {
-  return mView->formatter();
+  return mView->grantleeFormatter();
 }
 
 View *SearchEngine::view() const
@@ -411,6 +306,7 @@ void SearchEngine::finishSearch()
 {
   delete mRootTraverser;
   mRootTraverser = 0;
+  mSearchRunning = false;
 
   emit searchFinished();
 }
